import { EVMBlockchainIncludingTestnet } from '@crossmint/common-sdk-base';
export { EVMBlockchainIncludingTestnet as Blockchain, blockchainToChainId } from '@crossmint/common-sdk-base';
import { KernelSmartAccount } from '@zerodev/sdk';
import { SmartAccountClient } from 'permissionless';
import { SmartAccount } from 'permissionless/accounts';
import { EntryPoint } from 'permissionless/types/entrypoint';
import { LocalAccount, EIP1193Provider, Hex, HttpTransport, Chain, PublicClient } from 'viem';
import { TORUS_NETWORK_TYPE } from '@web3auth/single-factor-auth';

type SmartWalletSDKInitParams = {
    clientApiKey: string;
};
type UserParams = {
    /**
     * A unique identifier for the user. This must match the value of the identifier within the JWT
     * that is specified in the project settings (typically `sub`).
     */
    id: string;
    jwt: string;
};
type Web3AuthSigner = {
    type: "WEB3_AUTH";
    clientId: string;
    verifierId: string;
    web3AuthNetwork: TORUS_NETWORK_TYPE;
    jwt: string;
};
type ViemAccount = {
    type: "VIEM_ACCOUNT";
    account: LocalAccount & {
        source: "custom";
    };
};
type PasskeySigner = {
    type: "PASSKEY";
    /**
     * Displayed to the user during passkey registration or signing prompts.
     */
    passkeyName: string;
};
type EOASigner = EIP1193Provider | Web3AuthSigner | ViemAccount;
interface WalletConfig {
    signer: EOASigner | PasskeySigner;
}

declare const SUPPORTED_KERNEL_VERSIONS: readonly ["0.3.1", "0.3.0", "0.2.4"];
type SupportedKernelVersion = (typeof SUPPORTED_KERNEL_VERSIONS)[number];
declare const SUPPORTED_ENTRYPOINT_VERSIONS: readonly ["v0.6", "v0.7"];
type SupportedEntryPointVersion = (typeof SUPPORTED_ENTRYPOINT_VERSIONS)[number];
type PasskeyValidatorSerializedData = {
    passkeyServerUrl: string;
    credentials: string;
    entryPoint: Hex;
    validatorAddress: Hex;
    pubKeyX: string;
    pubKeyY: string;
    authenticatorIdHash: Hex;
};

type StoreSmartWalletParams = {
    type: string;
    smartContractWalletAddress: string;
    signerData: SignerData;
    sessionKeySignerAddress?: string;
    version: number;
    baseLayer: string;
    chainId: number;
    entryPointVersion: SupportedEntryPointVersion;
    kernelVersion: SupportedKernelVersion;
};
type SignerData = EOASignerData | PasskeySignerData;
interface EOASignerData {
    eoaAddress: string;
    type: "eoa";
}
type PasskeySignerData = PasskeyValidatorSerializedData & {
    passkeyName: string;
    domain: string;
    type: "passkeys";
};

declare class LoggerWrapper {
    private extraInfo;
    private logIdempotencyKey;
    constructor(className: string, extraInfo?: {}, logIdempotencyKey?: string);
    private logInput;
    private logOutput;
    private logError;
    protected logPerformance<T>(name: string, cb: () => Promise<T>): Promise<T>;
}

declare abstract class BaseCrossmintService extends LoggerWrapper {
    protected crossmintAPIHeaders: Record<string, string>;
    protected crossmintBaseUrl: string;
    private static urlMap;
    constructor(apiKey: string);
    protected fetchCrossmintAPI(endpoint: string, options: {
        body?: string;
        method: string;
    } | undefined, onServerErrorMessage: string, authToken?: string): Promise<any>;
    protected getUrlFromEnv(environment: string): string;
}

declare class CrossmintWalletService extends BaseCrossmintService {
    storeSmartWallet(user: UserParams, input: StoreSmartWalletParams): Promise<any>;
    getSmartWalletConfig(user: UserParams, chain: EVMBlockchainIncludingTestnet): Promise<any>;
    getPasskeySigner(user: UserParams): Promise<PasskeySignerData | null>;
    fetchNFTs(address: string, chain: EVMBlockchainIncludingTestnet): Promise<any>;
    getPasskeyServerUrl(user: UserParams): string;
}

interface EVMToken {
    chain: EVMBlockchainIncludingTestnet;
    contractAddress: string;
}
interface NFTEVMToken extends EVMToken {
    tokenId: string;
    type: "nft";
}
interface SFTEVMToken extends EVMToken {
    tokenId: string;
    type: "sft";
}
interface ERC2OEVMToken extends EVMToken {
    type: "ft";
}
type ERC20TransferType = {
    token: ERC2OEVMToken;
    amount: bigint;
};
type SFTTransferType = {
    token: SFTEVMToken;
    quantity: number;
};
type NFTTransferType = {
    token: NFTEVMToken;
};
type TransferType = ERC20TransferType | SFTTransferType | NFTTransferType;

/**
 * Smart wallet interface for EVM chains enhanced with Crossmint capabilities.
 * Core functionality is exposed via [viem](https://viem.sh/) clients within the `client` property of the class.
 */
declare class EVMSmartWallet extends LoggerWrapper {
    private readonly crossmintService;
    readonly chain: EVMBlockchainIncludingTestnet;
    /**
     * [viem](https://viem.sh/) clients that provide an interface for core wallet functionality.
     */
    readonly client: {
        /**
         * An interface to interact with the smart wallet, execute transactions, sign messages, etc.
         */
        wallet: SmartAccountClient<EntryPoint, HttpTransport, Chain, SmartAccount<EntryPoint>>;
        /**
         * An interface to read onchain data, fetch transactions, retrieve account balances, etc. Corresponds to public [JSON-RPC API](https://ethereum.org/en/developers/docs/apis/json-rpc/) methods.
         */
        public: PublicClient;
    };
    private readonly kernel;
    constructor(crossmintService: CrossmintWalletService, account: KernelSmartAccount<EntryPoint, HttpTransport>, publicClient: PublicClient<HttpTransport>, chain: EVMBlockchainIncludingTestnet);
    /**
     * The address of the smart wallet.
     */
    get address(): `0x${string}`;
    /**
     * @returns The transaction hash.
     */
    transfer(toAddress: string, config: TransferType): Promise<string>;
    /**
     * @returns A list of NFTs owned by the wallet.
     */
    nfts(): Promise<any>;
}

declare class NotAuthorizedError extends Error {
    code: string;
    constructor(message: string);
}
declare class TransferError extends Error {
    code: string;
    constructor(message: string);
}
declare class TransactionError extends Error {
    code: string;
    constructor(message: string);
}
declare class CrossmintServiceError extends Error {
    code: string;
    status?: number;
    constructor(message: string, status?: number);
}
declare class RunningOnServerError extends Error {
    readonly code = "ERROR_RUNNING_ON_SERVER";
    constructor();
}
/**
 * Generic undefined error
 */
declare class WalletSdkError extends Error {
    code: string;
    constructor(message: string);
}

declare class SmartWalletSDK extends LoggerWrapper {
    private readonly smartWalletService;
    private constructor();
    /**
     * Initializes the SDK with the **client side** API key obtained from the Crossmint console.
     * @throws error if the api key is not formatted correctly.
     */
    static init({ clientApiKey }: SmartWalletSDKInitParams): SmartWalletSDK;
    getOrCreateWallet(user: UserParams, chain: EVMBlockchainIncludingTestnet, walletConfig: WalletConfig): Promise<EVMSmartWallet>;
}

export { CrossmintServiceError, type EOASigner, type ERC20TransferType, EVMSmartWallet, type NFTTransferType, NotAuthorizedError, type PasskeySigner, RunningOnServerError, type SFTTransferType, SmartWalletSDK, type SmartWalletSDKInitParams, TransactionError, TransferError, type TransferType, type UserParams, type ViemAccount, type WalletConfig, WalletSdkError, type Web3AuthSigner };
